@@ -5,7 +5,6 @@ import random
 import re
 import string
 import uuid
-import json
 from collections import OrderedDict
 from datetime import datetime
 
@@ -72,13 +71,9 @@ def validate_info(info):
     return True
 
 
-def parse_xml(inputfile):
-    if not inputfile or not os.path.isfile(inputfile):
-        return None
-    with open(inputfile, encoding='utf-8-sig') as f:
-        xml_string = f.read()
-        inputfile_obj = xmltodict.parse(xml_string)
-        return inputfile_obj
+def parse_xml_to_dict(xml_string):
+    d = xmltodict.parse(xml_string)
+    return d
 
 
 def validate_wzdx(wzdx_obj, wzdx_schema):
@@ -110,7 +105,7 @@ def initialize_info(feed_info_id):
 
 # Add ids to message
 # This function may fail if some optional fields are not present (lanes, types_of_work, relationship, ...)
-def add_ids(message):
+def add_ids_v3(message):
     if not message or type(message) != dict:
         return None
 
@@ -133,7 +128,37 @@ def add_ids(message):
     return message
 
 
-def initialize_wzdx_object(info):
+# Add ids to message
+# This function may fail if some optional fields are not present (lanes, types_of_work, relationship, ...)
+def add_ids_v4(message, event_type):
+    if not message or type(message) != dict:
+        return None
+
+    if event_type == 'work-zone':
+        data_source_id = message.get('road_event_feed_info').get(
+            'data_sources')[0].get('data_source_id')
+    elif event_type == 'restriction':
+        data_source_id = message.get('feed_info').get(
+            'data_sources')[0].get('data_source_id')
+
+    road_event_length = len(message.get('features'))
+    road_event_ids = []
+    for i in range(road_event_length):
+        road_event_ids.append(str(uuid.uuid4()))
+
+    for i in range(road_event_length):
+        feature = message.get('features')[i]
+        id = road_event_ids[i]
+        feature['properties']['core_details']['road_event_id'] = id
+        feature['properties']['core_details']['data_source_id'] = data_source_id
+        if feature['properties']['core_details'].get('relationship'):
+            feature['properties']['core_details']['relationship']['relationship_id'] = str(
+                uuid.uuid4())
+            feature['properties']['core_details']['relationship']['road_event_id'] = id
+    return message
+
+
+def initialize_wzdx_object_v3(info):
     wzd = {}
     wzd['road_event_feed_info'] = {}
     # hardcode
@@ -182,6 +207,104 @@ def initialize_wzdx_object(info):
     return wzd
 
 
+def initialize_wzdx_object_v4(info):
+    wzd = {}
+    wzd['road_event_feed_info'] = {}
+    # hardcode
+    wzd['road_event_feed_info']['feed_info_id'] = info.get('feed_info_id')
+    wzd['road_event_feed_info']['update_date'] = datetime.utcnow().strftime(
+        "%Y-%m-%dT%H:%M:%SZ")
+    wzd['road_event_feed_info']['publisher'] = info.get(
+        'metadata').get('issuing_organization')
+    wzd['road_event_feed_info']['contact_name'] = info.get(
+        'metadata').get('contact_name')
+    wzd['road_event_feed_info']['contact_email'] = info.get(
+        'metadata').get('contact_email')
+    if info['metadata'].get('datafeed_frequency_update', False):
+        wzd['road_event_feed_info']['update_frequency'] = info.get('metadata')[
+            'datafeed_frequency_update']  # Verify data type
+    wzd['road_event_feed_info']['version'] = '4.0'
+    wzd['road_event_feed_info']['license'] = "https://creativecommons.org/publicdomain/zero/1.0/"
+
+    data_source = {}
+    data_source['data_source_id'] = str(uuid.uuid4())
+    data_source['feed_info_id'] = info.get('feed_info_id')
+    data_source['organization_name'] = info.get(
+        'metadata').get('issuing_organization')
+    data_source['contact_name'] = info.get('metadata').get('contact_name')
+    data_source['contact_email'] = info.get('metadata').get('contact_email')
+    if info['metadata'].get('datafeed_frequency_update', False):
+        data_source['update_frequency'] = info.get(
+            'metadata').get('datafeed_frequency_update')
+    data_source['update_date'] = datetime.utcnow().strftime(
+        "%Y-%m-%dT%H:%M:%SZ")
+    data_source['location_method'] = info.get(
+        'metadata').get('wz_location_method')
+    data_source['lrs_type'] = info.get('metadata').get('lrs_type')
+    wzd['road_event_feed_info']['data_sources'] = [data_source]
+
+    wzd['type'] = 'FeatureCollection'
+    sub_identifier = ''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for _ in
+                             range(6))  # Create random 6 character digit/letter string
+    id = str(uuid.uuid4())
+    ids = {}
+    ids['sub_identifier'] = sub_identifier
+    ids['id'] = id
+
+    wzd['features'] = []
+
+    return wzd
+
+
+def initialize_wzdx_object_restriction(info):
+    wzd = {}
+    wzd['feed_info'] = {}
+    # hardcode
+    wzd['feed_info']['feed_info_id'] = info.get('feed_info_id')
+    wzd['feed_info']['update_date'] = datetime.utcnow().strftime(
+        "%Y-%m-%dT%H:%M:%SZ")
+    wzd['feed_info']['publisher'] = info.get(
+        'metadata').get('issuing_organization')
+    wzd['feed_info']['contact_name'] = info.get(
+        'metadata').get('contact_name')
+    wzd['feed_info']['contact_email'] = info.get(
+        'metadata').get('contact_email')
+    if info['metadata'].get('datafeed_frequency_update', False):
+        wzd['feed_info']['update_frequency'] = info.get('metadata')[
+            'datafeed_frequency_update']  # Verify data type
+    wzd['feed_info']['version'] = '4.0'
+    wzd['feed_info']['license'] = "https://creativecommons.org/publicdomain/zero/1.0/"
+
+    data_source = {}
+    data_source['data_source_id'] = str(uuid.uuid4())
+    data_source['feed_info_id'] = info.get('feed_info_id')
+    data_source['organization_name'] = info.get(
+        'metadata').get('issuing_organization')
+    data_source['contact_name'] = info.get('metadata').get('contact_name')
+    data_source['contact_email'] = info.get('metadata').get('contact_email')
+    if info['metadata'].get('datafeed_frequency_update', False):
+        data_source['update_frequency'] = info.get(
+            'metadata').get('datafeed_frequency_update')
+    data_source['update_date'] = datetime.utcnow().strftime(
+        "%Y-%m-%dT%H:%M:%SZ")
+    data_source['location_method'] = info.get(
+        'metadata').get('wz_location_method')
+    data_source['lrs_type'] = info.get('metadata').get('lrs_type')
+    wzd['feed_info']['data_sources'] = [data_source]
+
+    wzd['type'] = 'FeatureCollection'
+    sub_identifier = ''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for _ in
+                             range(6))  # Create random 6 character digit/letter string
+    id = str(uuid.uuid4())
+    ids = {}
+    ids['sub_identifier'] = sub_identifier
+    ids['id'] = id
+
+    wzd['features'] = []
+
+    return wzd
+
+
 def string_to_number(field):
     try:
         return int(field)
@@ -212,27 +335,8 @@ def parse_direction_from_street_name(street):
     return direction
 
 
-def get_wzdx_schema(schema_file_name):
-    try:
-        schema_string = open(schema_file_name, 'r').read()
-    except FileNotFoundError as e:
-        raise RuntimeError('invalid schema: file does not exist')
-    if not schema_string:
-        raise RuntimeError('invalid schema: empty schema')
-    try:
-        schema_obj = json.loads(schema_string)
-        jsonschema.validate('', schema_obj)
-    except json.decoder.JSONDecodeError as e:
-        raise RuntimeError('invalid schema: not valid json') from e
-    except jsonschema.SchemaError as e:
-        raise RuntimeError('invalid schema: schema failed validation') from e
-    except jsonschema.ValidationError:
-        return schema_obj
-    return schema_obj
-
-
 # function to parse polyline to geometry line string
-def parse_polyline(poly):
+def parse_polyline_from_linestring(poly):
     if not poly or type(poly) != str:
         return None
     poly = poly[len('LINESTRING ('): -1]
